#include "main.h"

#define SONDERZEICHEN_CNT 5
const char Sonder_Z1[SONDERZEICHEN_CNT][8] PROGMEM = {{1,1,2,18,26,10,12,0},{0,17,10,4,10,17,0,0},{17,14,17,17,17,17,14,0},{10,17,17,17,17,17,14,0},{17,14,17,17,31,17,17,0}};

struct Display D1;

void trigger_EN(void) //EN wird getriggert
{
	DI_PORT |= 1<<E;
	asm("nop");
	asm("nop");
	asm("nop");
	DI_PORT &= ~(1<<E);
	asm("nop");
	asm("nop");
	asm("nop");
}

void Init_Sonderzeichen(void)
{
	unsigned char a,pos,cache;
	
	for (pos=0;pos<SONDERZEICHEN_CNT;pos++){
		for (a=0;a<8;a++){
			//Setze Adresse
			DisplaySendData(D_SET_SONDERZEICHEN_ADDR+pos*8+a);
			DI_PORT = 1<<RS;
			memcpy_P(&cache,Sonder_Z1[pos]+a,1);
			DisplaySendData(cache);	
			DI_PORT = 0;
		}
	}
	
	//Erfolgt nicht die Adressierung der Position, wird bei der Textaustgabe weiterhin der Sonderzeichenspeicher beschreiben.
	Setze_Aktuelle_Position();
}

unsigned char ConvertDData(unsigned char data)
{
	unsigned char cache;
	cache = ((data>>0)&1)<<D4;
	cache |= ((data>>1)&1)<<D5;
	cache |= ((data>>2)&1)<<D6;
	cache |= ((data>>3)&1)<<D7;
	return cache;
}

void Set_Adjustments(unsigned char Position, unsigned char value)
{
	if (value){
		D1.Adjustment |= Position;
	}
	else{
		D1.Adjustment &= ~Position;
	}
	DisplaySendData(D1.Adjustment);
}



void WaitD()
{
	unsigned char tmp = DI_PORT;
	
	DI_PORT = 1<<RW;
	DI_PORT_INI = ~(1<<D7);
	DI_PORT |= 1<<E;//Low-High-Flanke: Das Display erhlt den Befehl zur Ausgabe vom Busy bit
	asm("nop");
	asm("nop");
	while ((DI_PORT_INP&(1<<D7))!=0){asm("nop");};
	DI_PORT &= ~(1<<E);//Hiermit wird der Lesemodus wieder deaktiviert.
	trigger_EN();
	DI_PORT = tmp;//ursprnglicher Zustand wird wiederhergestellt
	DI_PORT_INI = 255;
}

void DisplaySendData(unsigned char data)
{	
	
	WaitD();
	DI_PORT &= 240;
	DI_PORT |= ConvertDData(data>>4);
	trigger_EN();
	DI_PORT &= 240;
	DI_PORT |= ConvertDData(data);
	trigger_EN();
	DI_PORT &= 240;
}

void Setze_Aktuelle_Position(void)
{
	unsigned char tmp=0;
	
	switch(D1.Zeilen_Pos){
		case 0:{
			tmp = D_Z1_ADDR;
		}break;
		case 1:{
			tmp = D_Z2_ADDR;
		}break;
		case 2:{
			tmp = D_Z3_ADDR;
		}break;
		case 3:{
			tmp = D_Z4_ADDR;
		}break;
	}
	tmp += D1.Spalten_Pos;
	DisplaySendData(D_SET_POSITION_ADDR+tmp);
}

void D_Check_Boundarys(unsigned char Pos_changed)
{	
	if (D1.Spalten_Pos>=D_SPALTEN_CNT){
		D1.Spalten_Pos = 0;
		D1.Zeilen_Pos++;
		Pos_changed = 1;
	}
	if (D1.Zeilen_Pos>=D_ZEILEN_CNT){
		D1.Zeilen_Pos = 0;
		D1.Spalten_Pos = 0;
		Pos_changed = 1;
	}
	if (Pos_changed){
		Setze_Aktuelle_Position();
	}
}

void Setzte_Position(unsigned char x, unsigned char y)
{
	D1.Spalten_Pos = x;
	D1.Zeilen_Pos = y;
	Setze_Aktuelle_Position();
}

void D_Add_Char(unsigned char c)
{

	if (c == '\n'){
		while (D1.Spalten_Pos>0)
		{
			DI_PORT = 1<<RS;
			DisplaySendData(' ');
			DI_PORT = 0;			
			D1.Spalten_Pos++;
			D_Check_Boundarys(1);
		}
	}
	else if ((c>31)||(c<8)){
		DI_PORT = 1<<RS;
		
		if (c=='') c = 225;
		else if (c=='') c = 245;
		else if (c=='') c = 239;
		else if (c=='') c = 226;
		else if (c=='') c = 4;
		else if (c=='') c = 3;
		else if (c=='') c = 2;	
		DisplaySendData(c);
		DI_PORT = 0;		
		D1.Spalten_Pos++;
		D_Check_Boundarys(0);
	}
}

void DText(unsigned char *Text)
{
	unsigned char a;
	a = 0;
	while (Text[a]!=0){
		D_Add_Char(Text[a]);
		a++;
	}
}



void init_display()
{
	unsigned char z;
	DI_PORT_INI = 255;
	DI_PORT = 0;
	BACKLIGHT_P_INI |= 1<<BACKLIGHT_PIN;
	Delay1ms(40);//20ms
	
	for(z=0;z<3;z++){ 
		DI_PORT = ConvertDData(48>>4);
		trigger_EN();
		Delay1ms(10);//5ms
	}
	DI_PORT = ConvertDData(32>>4); // Datenbus wird auf 4 bit gesetzt
	trigger_EN();	
	DisplaySendData(32); //Mehrzeiliges(44)/Einzeiliges(32) Display wird eingestellt 
	DisplaySendData(8); //Dsiplay off.
	DisplaySendData(1); //Displayinhalt lschen.
	Set_Adjustments(DIADJUSTCONST + DISPLAY_ON + CURSOR_ON + BLINKING_ON,1);
	DisplaySendData(6); //Text soll nicht verschoben werden, Curser bewegt sich nach rechts.
	DisplaySendData(2); //Der Cursor wird an die Startposition gebracht
	DisplaySendData(1); //Displayinhalt lschen.
	
	Init_Sonderzeichen();
}
